use strict;
use Test::More skip_all => "TODO";
# use Test::Requires 'Test::Pod::Coverage';
# plan tests => 1;
# Test::Pod::Coverage::pod_coverage_ok( "XML::RSS::LibXML", "XML::RSS::LibXML is covered" );
